@@ -6,6 +6,7 @@ function help {
   echo "t/testc.sh [OPTIONS] [1-$ntests]"
   echo " -D<debugflags>     for O=C or O=CC. Default: C,-DcOACMSGpu,-v resp. CC,-DoOscprSql,-v"
   echo " -O<0-4>            optimization level"
+  echo " -f<opt>            special optimization"
   echo " -B<static|dynamic> pass to cc_harness"
   echo " -c                 continue on errors"
   echo " -k                 keep temp. files on PASS"
@@ -39,15 +40,11 @@ else
         OCMD="$PERL $Mblib -MO=CC,-DoOscprSql,-v,"
     fi
 fi
-OCMDO1="$(echo $OCMD|sed -e s/C,-D/C,-O1,-D/)"
-OCMDO2="$(echo $OCMD|sed -e s/C,-D/C,-O2,-D/)"
-OCMDO3="$(echo $OCMD|sed -e s/C,-D/C,-O3,-D/)"
-OCMDO4="$(echo $OCMD|sed -e s/C,-D/C,-O4,-D/)"
 CONT=
 # 5.6: rather use -B static
 #CCMD="$PERL script/cc_harness -g3"
 # rest. -DALLOW_PERL_OPTIONS for -Dtlv
-CCMD="$PERL script/cc_harness -d -g3 -Bdynamic -DALLOW_PERL_OPTIONS"  
+CCMD="$PERL $Mblib script/cc_harness -g3 -DALLOW_PERL_OPTIONS"  
 LCMD=
 # On some perls I also had to add $archlib/DynaLoader/DynaLoader.a to libs in Config.pm
 }
@@ -75,17 +72,12 @@ function fail {
 function runopt {
     o=$1
     optim=$2
+    OCMDO1="$(echo $OCMD|sed -e s/C,-D/C,-O$optim,-D/)"
     suff="_o${optim}"
     if [ "$optim" == "0" ]; then suff=""; fi
     rm ${o}${suff} ${o}${suff}.c 2> /dev/null
-    if [ $optim == 1 ]; then CMD=$OCMDO1
-     else if [ $optim == 2 ]; then CMD=$OCMDO2
-      else if [ $optim == 3 ]; then CMD=$OCMDO3
-       else if [ $optim == 4 ]; then CMD=$OCMDO4
-        else CMD=$OCMD
-       fi
-      fi
-     fi
+    if [ $optim -lt 5 ]; then CMD=$OCMDO1
+    else CMD=$OCMD
     fi
     vcmd ${CMD}-o${o}${suff}.c $o.pl
     test -z $CPP || vcmd $CCMD ${o}${suff}.c -c -E -o ${o}${suff}_E.c
@@ -149,15 +141,15 @@ function ctest {
     fi
 }
 
-ntests=39
+ntests=45
 declare -a tests[$ntests]
 declare -a result[$ntests]
 ncctests=3
 declare -a cctests[$((100+$ncctests))]
 declare -a ccresult[$((100+$ncctests))]
-tests[1]="print 'hi'"
+tests[1]='print "hi"'
 result[1]='hi';
-tests[2]="for (1,2,3) { print if /\d/ }"
+tests[2]='for (1,2,3) { print if /\d/ }'
 result[2]='123';
 tests[3]='$_ = "xyxyx"; %j=(1,2); s/x/$j{print("z")}/ge; print $_'
 result[3]='zzz2y2y2';
@@ -228,7 +220,7 @@ result[25]="0 1 2 3`$PERL -e'print (($] < 5.007) ? q( 4 5) : q())'` 4321";
 tests[26]='sub a:lvalue{my $a=26; ${\(bless \$a)}}sub b:lvalue{${\shift}}; print ${a(b)}';
 result[26]="26";
 # xsub constants
-tests[27]='use Fcntl; print "ok" if ( &Fcntl::O_WRONLY );'
+tests[27]='use Fcntl (); print "ok" if ( Fcntl::O_CREAT() == 64 && &Fcntl::O_CREAT == 64 ); '
 result[27]='ok'
 # require $fname
 tests[28]='my($fname,$tmp_fh);while(!open($tmp_fh,">",($fname=q{cctest28_} . rand(999999999999)))){$bail++;die "Failed to create a tmp file after 500 tries" if $bail>500;}print {$tmp_fh} q{$x="ok";1;};close($tmp_fh);require $fname;unlink($fname);print $x;'
@@ -267,11 +259,32 @@ result[37]='ok'
 tests[38]='for(1 .. 1024) { if (open(my $null_fh,"<","/dev/null")) { seek($null_fh,0,SEEK_SET); close($null_fh); $ok++; } }if ($ok == 1024) { print "ok"; }'
 result[38]='ok'
 # check re::is_regexp, and on 5.12 if being upgraded to SVt_REGEXP
+# => Undefined subroutine &re::is_regexp with B-C-1.19, even with -ure
 usere="`$PERL -e'print (($] < 5.011) ? q(use re;) : q())'`"
 tests[39]='{'$usere'$a=${qr//};$a=2;print ($] < 5.007?1:re::is_regexp(\$a))}'
 result[39]='1'
-# => Undefined subroutine &re::is_regexp with B-C-1.19, even with -ure
-
+# String with a null byte -- used to generate broken .c on 5.6.2 with static pvs
+tests[40]='my $var="this string has a null \\000 byte in it";print "ok";'
+result[40]='ok'
+# Shared scalar, n magic. => Don't know how to handle magic of type \156.
+usethreads="`$PERL -MConfig -e'print ($Config{useithreads} ? q(use threads;) : q())'`"
+#usethreads='BEGIN{use Config; unless ($Config{useithreads}) {print "ok"; exit}} '
+#;threads->create(sub{$s="ok"})->join;
+# not yet testing n, only P
+tests[41]=$usethreads'use threads::shared;{my $s="ok";share($s);print $s}'
+result[41]='ok'
+# Shared aggregate, P magic
+tests[42]=$usethreads'use threads::shared;my %h : shared; print "ok"'
+result[42]='ok'
+# Aggregate element, n + p magic
+tests[43]=$usethreads'use threads::shared;my @a : shared; $a[0]="ok"; print $a[0]'
+result[43]='ok'
+# perl #72922 (5.11.4 fails with magic_killbackrefs)
+tests[44]='use Scalar::Util "weaken";my $re1=qr/foo/;my $re2=$re1;weaken($re2);print "ok" if $re3=qr/$re1/;'
+result[44]='ok'
+# test autoload and xs_init
+tests[45]='use Data::Dumper ();Data::Dumper::Dumper(\%main::);print "ok";'
+result[45]='ok'
 
 # from here on we test CC specifics only
 
@@ -289,8 +302,8 @@ result[103]='B::PV'
 init
 
 # 
-# getopts for -q -k -E -Du,-q -v -O2, -a -c
-while getopts "hqackoED:B:O:" opt
+# getopts for -q -k -E -Du,-q -v -O2, -a -c -fro-inc
+while getopts "hqackoED:B:O:f:" opt
 do
   if [ "$opt" = "q" ]; then 
     QUIET=1
@@ -302,25 +315,31 @@ do
     OCMDO2="$(echo $OCMDO2|sed -e 's/-D.*,//' -e 's/,-v,/,/' -e s/-MO=/-MO=$qq/)"
     OCMDO3="$(echo $OCMDO3|sed -e 's/-D.*,//' -e 's/,-v,/,/' -e s/-MO=/-MO=$qq/)"
     OCMDO4="$(echo $OCMDO4|sed -e 's/-D.*,//' -e 's/,-v,/,/' -e s/-MO=/-MO=$qq/)"
-    CCMD="$PERL script/cc_harness -q -g3 -Bdynamic -DALLOW_PERL_OPTIONS"
+    CCMD="$PERL $Mblib script/cc_harness -q -g3 -DALLOW_PERL_OPTIONS"
   fi
   if [ "$opt" = "o" ]; then Mblib=" "; init; fi
   if [ "$opt" = "c" ]; then CONT=1; fi
   if [ "$opt" = "k" ]; then KEEP=1; fi
   if [ "$opt" = "E" ]; then CPP=1; fi
   if [ "$opt" = "h" ]; then help; exit; fi
-  # -D options: u,-q for quiet, no -D for verbose
+  # -D options: u,-q for quiet, no -D for verbose, -D- for no gcc warnings
   if [ "$opt" = "D" ]; then
     OCMD="$PERL $Mblib -MO=C,-D${OPTARG},"
     if [ $BASE = "testcc.sh" ]; then 
         OCMD="$PERL $Mblib -MO=CC,-D${OPTARG},"
     fi
+    if [ -z "${OPTARG/-/}" ]; then
+	CCMD="$PERL $Mblib script/cc_harness -d -g3 -DALLOW_PERL_OPTIONS"  
+    fi
   fi
   # -B dynamic or -B static
   if [ "$opt" = "B" ]; then 
-    CCMD="$PERL script/cc_harness -d -g3 -B${OPTARG} -DALLOW_PERL_OPTIONS"
+    CCMD="$PERL $Mblib script/cc_harness -d -g3 -B${OPTARG} -DALLOW_PERL_OPTIONS"
   fi
   if [ "$opt" = "O" ]; then OPTIM="$OPTARG"; fi
+  if [ "$opt" = "f" ]; then
+    OCMD="$(echo $OCMD|sed -e "s/C,/C,-f$OPTARG,/")"
+  fi
   if [ "$opt" = "a" ]; then # replace -Du, by -Do
     OCMD="$(echo $OCMD|sed -r -e 's/(-D.*)u,/\1o,/')" 
   fi
