@@ -11,6 +11,7 @@
 function help {
   echo "t/testm.sh [OPTIONS] [module|modules-file]..."
   echo " -k                 keep temp. files on PASS"
+  echo " -D<arg>            add debugging flags"
   echo " -l                 log"
   echo " -o                 orig. no -Mblib, use installed modules (5.6, 5.8)"
   echo " -t                 run the module tests also, not only use Module (experimental)"
@@ -42,12 +43,14 @@ function fail {
     echo
 }
 
-while getopts "hoklts" opt
+while getopts "hokltTsD:" opt
 do
   if [ "$opt" = "o" ]; then Mblib=" "; init; fi
   if [ "$opt" = "k" ]; then KEEP="-S"; fi
+  if [ "$opt" = "D" ]; then KEEP="-D${OPTARG}"; fi
   if [ "$opt" = "l" ]; then TEST="-log"; fi
   if [ "$opt" = "t" ]; then TEST="-t"; fi
+  if [ "$opt" = "T" ]; then PERLCC_OPTS="--time"; PERLCC_TIMEOUT=120; fi
   if [ "$opt" = "s" ]; then 
       v=$($PERL -It -Mmodules -e'print perlversion')
       if [ -f log.modules-$v ]; then # and not older than a few days
@@ -69,6 +72,7 @@ fi
 # need to shift the options
 while [ -n "$1" -a "${1:0:1}" = "-" ]; do shift; done
 
+PERLCC_TIMEOUT=120
 if [ -n "$1" ]; then
     if [ -f "$1" ]; then
 	# run a mymodules.t like test
@@ -77,8 +81,21 @@ if [ -n "$1" ]; then
 	while [ -n "$1" ]; do
 	    # single module
 	    name="$(perl -e'$ARGV[0]=~s{::}{_}g; print lc($ARGV[0])' $1)"
-	    echo $PERL $Mblib blib/script/perlcc -r $KEEP -e "\"use $1; print 'ok'\"" -o $name
-	    $PERL $Mblib blib/script/perlcc -r $KEEP -e "use $1; print 'ok'" -o $name
+	    if [ "${KEEP:0:2}" = "-D" ]; then
+	      echo $PERL $Mblib -MO=C,$KEEP,-o$name.c -e "\"use $1; print 'ok'\""
+	      $PERL $Mblib -MO=C,$KEEP,-o$name.c -e "use $1; print 'ok'"
+	      if [ -f $name.c ]; then
+		echo $PERL $Mblib script/cc_harness -d -g3 -o$name $name.c
+		$PERL $Mblib script/cc_harness -d -g3 -o$name $name.c
+		if [ -f $name ]; then
+		  echo "running ./$name"
+		  ./$name
+		fi
+	      fi
+	    else
+	      echo $PERL $Mblib blib/script/perlcc $PERLCC_OPTS -r $KEEP -e "\"use $1; print 'ok'\"" -o $name
+	      $PERL $Mblib blib/script/perlcc $PERLCC_OPTS -r $KEEP -e "use $1; print 'ok'" -o $name
+            fi
 	    test -f a.out.c && mv a.out.c $name.c
 	    [ -n "$TEST" ] && $PERL $Mblib -It -MCPAN -Mmodules -e"CPAN::Shell->testcc(q($1))"
 	    shift
